/*
  xsetptrans - pseudo-transparent property setting utility for X.
  Copyright (C) 2002-2004 KATO Kazuyoshi

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Id: xsetptrans.c,v 1.5 2004/07/06 10:09:14 kzys Exp $
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <unistd.h>

char* progname;

void usage(void);
void version(void);
int rootToPixmap(Display* display,
		 Window root, Pixmap pixmap, GC gc, int width, int height);
void deleteTransProperty(Display* display, Window root);
void setTransProperty(Display* display, Window root, Pixmap pixmap);

int
main(int argc, char* argv[])
{
    Display* display;
    int screenNo;
    Window root;
    Pixmap pixmap;
    GC gc;
    XGCValues gcValues;
    Screen* screen;
    int c;
    char* displayName = NULL;


    progname = argv[0];

    while ((c = getopt(argc, argv, "d:hv")) != -1) {
	switch (c) {
	case 'd':
	    displayName = optarg;
	    break;
	case 'v':
	    version();
	    return 0;
	case 'h':
	    usage();
	    return 0;
	default:
	    usage();
	    return 1;
	}
    }

    display = XOpenDisplay(displayName);
    if (display == NULL) {
        fprintf(stderr, "%s: Could't open display %s.\n",
		progname,
		(displayName != NULL)? displayName : "");
        return 1;
    }
    screenNo = DefaultScreen(display);
    root = RootWindow(display, screenNo);
    screen = ScreenOfDisplay(display, screenNo);


    pixmap = XCreatePixmap(display, root, screen->width, screen->height,
                           DefaultDepth(display, screenNo));
    gcValues.foreground = WhitePixel(display, screenNo);
    gc = XCreateGC(display, pixmap, GCForeground, &gcValues);
    XFillRectangle(display, pixmap, gc, 0, 0, screen->width, screen->height);

    if (! rootToPixmap(display,
		       root, pixmap, gc, screen->width, screen->height)) {
        fprintf(stderr, "%s: Could't get root window image.\n", progname);
        return 1;
    }
    deleteTransProperty(display, root);
    setTransProperty(display, root, pixmap);

    XCloseDisplay(display);

    return 0;
}

void
usage()
{
    printf("Usage: %s [OPTIONS]\n", progname);
    printf("  -d DISPLAY  use display connection\n"
	   "  -h          display this help and exit\n"
	   "  -v          output version information and exit\n");
}

void
version()
{
    printf("xsetptrans " VERSION "\n");
}

/* This function is based on the mlterm <http://mlterm.sf.net/> */
int
rootToPixmap(Display* display,
	     Window root, Pixmap pixmap, GC gc, int width, int height)
{
    Window window;
    XSetWindowAttributes attrs;
    XEvent event;
    int count;

    attrs.background_pixmap = ParentRelative ;
    attrs.backing_store = Always ;
    attrs.event_mask = ExposureMask ;
    attrs.override_redirect = True ;

    window = XCreateWindow(display, root,
                           0, 0, width, height, 0,
                           CopyFromParent, CopyFromParent, CopyFromParent,
                           CWBackPixmap | CWBackingStore |
			   CWOverrideRedirect | CWEventMask,
                           &attrs);
    XGrabServer(display);
    XMapRaised(display, window);
    XSync(display, False);

    count = 0;
    while (! XCheckWindowEvent(display, window, ExposureMask, &event)) {
	usleep(10000);
        count++;

        if(count >= 10){
	    XDestroyWindow(display, window);
	    XUngrabServer(display);

	    return  0;
        }
    }

    XCopyArea(display, window, pixmap, gc,
              0, 0, width, height, 0, 0);

    XDestroyWindow(display, window);
    XUngrabServer(display);
    return 1;
}

void
deleteTransProperty(Display* display, Window root)
{
    Atom xrootpmap_id, type;
    int format;
    unsigned long length, after;
    unsigned char* rootData;


    xrootpmap_id = XInternAtom(display, "_XROOTPMAP_ID", True);
    if (xrootpmap_id == None) {
        return;
    }

    if (XGetWindowProperty(display, root, xrootpmap_id,
			   0L, 1L, False, XA_PIXMAP,
			   &type, &format, &length, &after, &rootData)) {
        XFree(rootData);
    }
}

void
setTransProperty(Display* display, Window root, Pixmap pixmap)
{
    Atom xrootpmap_id;

    xrootpmap_id = XInternAtom(display, "_XROOTPMAP_ID", False);
    if (xrootpmap_id == None) {
        return;
    }

    XChangeProperty(display, root,
                    xrootpmap_id, XA_PIXMAP, 32, PropModeReplace,
                    (unsigned char *) &pixmap, 1);

    XSetCloseDownMode(display, RetainPermanent);
    return;
}
